package edu.thi.phga.aparapi_test;

import java.util.ArrayDeque;
import java.util.Deque;

import com.aparapi.Range;

import javafx.application.Application;
import javafx.geometry.Rectangle2D;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.image.ImageView;
import javafx.scene.image.PixelFormat;
import javafx.scene.image.PixelReader;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritableImage;
import javafx.scene.image.WritablePixelFormat;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.stage.Screen;
import javafx.stage.Stage;

public class Mandelbrot extends Application{
  private static Rectangle2D bounds = Screen.getPrimary().getBounds();
  private static final int   BREITE = (int) (bounds.getWidth() * 0.8);
  private static final int   HOEHE  = BREITE * 9 / 16;

  private double[] farbe      = new double[BREITE * HOEHE];
  private double[] saturation = new double[BREITE * HOEHE];

  private MandelbrotKernel k = new MandelbrotKernel(farbe, saturation);
  private Range r;

  // (x1, y1) = links oben, (x2, y2) = rechts unten
  private double x1 = -3.0;
  private double x2 = 1.8;
  private double y1 = (x2 - x1) * HOEHE / BREITE / 2;
  private double y2 = -y1;

  // Wie scharf ist das Bild (Ab wann sagen wir |Z_n| <= 2 für alle Z_n)
  private int maxIter = 200;
  private int exponent = 0;

  private WritableImage image = new WritableImage(BREITE, HOEHE);
  private WritablePixelFormat<java.nio.ByteBuffer> format = PixelFormat.getByteBgraPreInstance();

  private PixelWriter pw = image.getPixelWriter();
  private PixelReader pr = image.getPixelReader();

  private Deque<byte[]> imageStack = new ArrayDeque<>();
  private Deque<double[]> frameStack = new ArrayDeque<>();

  private Stage stage;

  private void zeichne() {
    stage.setTitle("KAO: Computing...");

    k.setParameters(x1, y1, x2, y2, maxIter);
    k.execute(r);

    for (int i = 0; i < farbe.length; i++) {
      Color c = Color.hsb(farbe[i], saturation[i], 0.5);
      pw.setColor(i % BREITE, i / BREITE, c);
    }

    setTitle();
  }

  private void setTitle() {
    stage.setTitle(String.format("KAO: Mandelbrot (Zoom: %d^%d, n = %d)",
        App.multiParam, exponent, maxIter));
  }

  private void zoomIn(MouseEvent e) {
    byte[] buffer = new byte[4 * BREITE * HOEHE];
    pr.getPixels(0, 0, BREITE, HOEHE, format, buffer, 0, 4 * BREITE);
    imageStack.push(buffer);
    frameStack.push(new double[] { x1, y1, x2, y2 });

    double x = x1 + (x2 - x1) * e.getX() / BREITE;
    double y = y1 + (y2 - y1) * e.getY() / HOEHE;

    double dx = (x2 - x1) / App.multiParam / 2;
    double dy = (y2 - y1) / App.multiParam / 2;

    x1 = x - dx;
    x2 = x + dx;
    y1 = y - dy;
    y2 = y + dy;

    maxIter *= 2;
    exponent++;

    zeichne();
  }

  private void zoomOut(KeyEvent e) {
    String key = e.getText();
    System.out.println("Key: " + key);
    if (key.equals("q") && !imageStack.isEmpty()) {
      double[] frame = frameStack.pop();
      x1 = frame[0];
      y1 = frame[1];
      x2 = frame[2];
      y2 = frame[3];

      maxIter /= 2;

      pw.setPixels(0, 0, BREITE, HOEHE, format, imageStack.pop(), 0, 4 * BREITE);

      setTitle();
    }
  }

  @Override
  public void start(Stage stage) {
    // Set range
    int maxWGSize = 16;
    System.out.println("Using WGSize: " + maxWGSize);

    r = Range.create2D(App.device, BREITE, HOEHE, maxWGSize, maxWGSize);

    this.stage = stage;
    ImageView view = new ImageView(image);
    view.setFocusTraversable(true);
    view.setOnMousePressed(this::zoomIn);
    view.setOnKeyPressed(this::zoomOut);

    zeichne();

    stage.setScene(new Scene(new Group(view)));
    stage.setResizable(false);
    stage.sizeToScene();
    stage.show();
  }
}
