package edu.thi.phga.aparapi_test;

import com.aparapi.Kernel;

public class MandelbrotKernel extends Kernel {
  private double[] farbe;
  private double[] saturation;
  private double x1, x2, y1, y2;
  private int maxIter;

  public MandelbrotKernel(double[] farbe, double[] saturation) {
    this.farbe = farbe;
    this.saturation = saturation;
  }

  public void setParameters(double x1, double y1, double x2, double y2, int maxIter) {
    this.x1 = x1;
    this.y1 = y1;

    this.x2 = x2;
    this.y2 = y2;

    this.maxIter = maxIter;
  }

  // Formel:
  // z_n = {             0, n=0
  //         z_(n-1)^2 + c, n>0
  // c element von C (Complex)
  // Bei uns ist c dann ein Pixel den wir einfärben
  // bzw. der Punkt(x, y) (offset) für eine bestimmte Zahl der Mandelbrot Menge
  // Schwarz == ist Teil der Menge (|z_k| <= 2)
  // Farbe   == sonst (|z_k| > 2) (Frühzeitiger Schleifenabbruch)
  // Alle z_n beginnen bei 0
  // Iter 1: z = 0^2 + c
  // Iter 2: z = c^2 + c
  // Iter 3: z = (c^2 + c)^2 + c
  // ...
  // Abbruch: |z| > 2 || Iter == maxIter (200)

  // @Override public void run() {
  //   int b = getGlobalId(0); // horizontaler Index
  //   int h = getGlobalId(1); // vertikaler Index

  //   double cx = x1 + (x2 - x1) * b / getGlobalSize(0);
  //   double cy = y1 + (y2 - y1) * h / getGlobalSize(1);

  //   double x  = cx   , y  = cy;
  //   double xx = x * x, yy = y * y;

  //   int i;

  //   for (i = 1; xx + yy < 4 && i <= maxIter; i++) {
  //     y  = 2 * x * y + cy;
  //     x  = xx - yy   + cx;
  //     yy = y * y;
  //     xx = x * x;
  //   }

  //   int idx         = h * getGlobalSize(0) + b;
  //   farbe[idx]      = log(i) * 360;
  //   helligkeit[idx] = i < maxIter ? 1 : 0;
  // }

  @Override
  public void run() {
    int x = getGlobalId(0);
    int y = getGlobalId(1);

    int sizeX = getGlobalSize(0);
    int sizeY = getGlobalSize(1);
    double imageSizeX = x2 - x1;
    double imageSizeY = y2 - y1;

    // Skalierung
    double cR = x1 + imageSizeX * x / sizeX;
    double cI = y1 + imageSizeY * y / sizeY;

    double zR = cR;
    double zI = cI;
    double zRQ = zR * zR;
    double zIQ = zI * zI;
    double zAbs = zIQ + zRQ;
    int i = 0;

    for (i = 1; i <= maxIter && zAbs < 4; i++) {
      // WICHTIG REIHENFOLGE!!! zR wird überschrieben...
      zI = 2 * zR * zI + cI; // 2 * xyi + c_imag
      zR = zRQ - zIQ + cR;   // x^2 - y^2 + c_real

      zIQ = zI * zI;
      zRQ = zR * zR;
      zAbs = zIQ + zRQ;
    }

    int index = y * sizeX + x;
    if (i < maxIter) {
      saturation[index] = min(i * log(i) / maxIter + 0.2, 1);
      farbe[index] = log(i) * 360;
    } else {
      saturation[index] = 0;
    }
  }
  // z = z*z + c
  // z^2 = (z_r + z_i*i) * (z_r + z_i*i)
  // = (z_r * z_r) + (z_r * z_i*i + z_r * z_i * i) - (z_i * z_i) <- [i^2 = -1]
  // = z_r^2 + 2*z_r*z_i*i - z_i^2
  // = z_r^2 - z_i^2 + 2*i*z_r*z_i
  // = x^2 - y^2 + 2xyi
  //   real      + imag

  // Nette Entdeckung :) #LaserShow WUBWUBWUBWUB
  // @Override
  // public void run() {
  //   int sizeX = getGlobalSize(0);
  //   int sizeY = getGlobalSize(1);
  //   double imageSizeX = (x2 - x1);
  //   double imageSizeY = (y2 - y1);
  //   int x = getGlobalId(0);
  //   int y = getGlobalId(1);
  //   double c_r = x + imageSizeX / sizeX;
  //   double c_i = y + imageSizeY / sizeY;
  //   double z_r = 0; // komplexe zahl
  //   double z_i = 0;
  //   double zb = 0;

  //   for (int i = 0; i < 200 && zb <= 4; i++) {
  //     // z = z*z + c
  //     // z^2
  //     z_r = z_r * z_r - z_i * z_i;
  //     z_i = z_r * z_i + z_r * z_i;

  //     // z + c;
  //     z_r += c_r;
  //     z_i += c_i;

  //     // sqrt(z_r^2 + z_i^2)
  //     zb = sqrt(z_r * z_r + z_i * z_i);
  //   }

  //   if (zb <= 4) {
  //     helligkeit[x + x * y] = 0;
  //   } else {
  //     helligkeit[x + x * y] = 1;
  //   }
  // }
}
