package edu.thi.phga.aparapi_test;

import com.aparapi.Kernel;
import com.aparapi.Range;

public class OpenCLReduce3 {
  public static void start(int[] a) {
    int optimalSize = a.length / 2;
    int N = a.length;
    int[] gs = new int[optimalSize];

    Kernel k = new Kernel() {
        @Override
        public void run() {
          int size = N / getGlobalSize(); // 2, 4, 8, ...
          int i = getGlobalId() * size; // 2: 0 -> 0, 1 -> 2, 2 -> 4, 3 -> 6, ...

          gs[getGlobalId()] = size;

          a[i] += a[i + size / 2]; // 1, 2, 4, ..
        }
      };

    // Manually manage variables that are accessible and returned by kernel
    // Anything that is passed to the constructor is still automatically pushed once
    // to the Memory on the gpu. This is done to reduce the amount of times
    // the data has to be transferred to the gpu memory via slow pci bus
    k.setExplicit(true);

    try {
      System.out.printf("MAX WG SIZE: %d\n", k.getKernelMaxWorkGroupSize(App.device));
    } catch(Exception e) {

    }
    // Not necessary (So maybe anything used by the kernel is put once?)
    // k.put(a);

    k.execute(Range.create(N / 2, optimalSize < 256 ? optimalSize : 256));
    double ct = k.getConversionTime();

    for (int n = N / 4; n >= 1; n /= 2) {
      k.execute(Range.create(n, n < 256 ? n : 256));
    }

    k.get(a);

    double et = k.getAccumulatedExecutionTime() - ct;
    System.out.printf("R3 - WANT: %d, GOT: %d, TIME: %.2f ms\n", a.length, a[0], et);

  }
}
