package edu.thi.phga.aparapi_test;

import com.aparapi.Range;

public class OpenCLReduce5 {
  public static void start(int[] a) {
    int N = a.length;

    int localMem  = (int) App.device.getLocalMemSize();
    int maxWGSize = Math.min(App.device.getMaxWorkGroupSize(), 256);

    int localSize  = Math.min(maxWGSize, localMem / 4);
    int globalSize = Math.min(N, 2 * localSize * localSize);
    localSize      = Math.min(localSize, globalSize);

    // These have to be created on the host side for the kernel to access them later
    // int[] a = new int[N] Folie 180 das input array
    int[] result = new int[2 * localSize]; // Folie 180
    int[] summe = { 0 }; // Folie 180

    OpenCLReduce5Kernel k = new OpenCLReduce5Kernel(a, localSize, result, summe);

    try {
      System.out.printf("KERNEL MAX WG SIZE: %d\n", k.getKernelMaxWorkGroupSize(App.device));
    } catch(Exception e) {

    }

    // Manually manage variables that are accessible and returned by kernel
    // Anything that is passed to the constructor is still automatically pushed once
    // to the Memory on the gpu. This is done to reduce the amount of times
    // the data has to be transferred to the gpu memory via slow pci bus
    k.setExplicit(true);

    Range r1 = Range.create(App.device, globalSize, localSize);
    Range r2 = Range.create(App.device, localSize, localSize);

    // Not necessary (So maybe anything used by the kernel is put once?)
    // k.put(a);

    k.execute(r1);
    double ct = k.getConversionTime();

    k.setStep(2);
    k.execute(r2);
    k.get(summe);

    double et = k.getAccumulatedExecutionTime() - ct;
    System.out.printf("R5 - WANT: %d, GOT: %d, TIME: %.2f ms\n", a.length, summe[0], et);

  }
}
