package edu.thi.phga.aparapi_test;

import java.util.Arrays;
import java.util.stream.IntStream;

import com.aparapi.Kernel;
import com.aparapi.Range;

import javafx.application.Application;
import javafx.geometry.Rectangle2D;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.layout.VBox;
import javafx.stage.Screen;
import javafx.stage.Stage;

public class Rauschfilter extends Application {
  private static Rectangle2D bounds = Screen.getPrimary().getBounds();

  private static final int BREITE = (int) (bounds.getWidth() * 0.8);
  private static final int HOEHE = BREITE * 9 / 16;

  private static final double OMEGA = 8 * Math.PI / BREITE;
  // private static final int FILTERLAENGE = 15;

  @Override
  public void start(Stage stage) {
    float[] noisy = new float[BREITE];
    float[] clean = new float[BREITE];
    Range r = Range.create(App.device, BREITE, Math.min(BREITE, 256));

    for (int i = 0; i < BREITE; i++) {
      noisy[i] = (float) (5 * Math.sin(i * OMEGA) + Math.random() - 0.5);
    }

    Kernel k = new RauschfilterKernel(noisy, clean, App.multiParam);
    System.out.println("Executing Kernel");
    k.execute(r);
    double ct = k.getConversionTime();
    double et = k.getExecutionTime() - ct;

    System.out.printf("Took %.2f ms to smooth out the wave with N = %d\n",
      et, App.multiParam);

    Canvas oben = getCanvas(noisy);
    Canvas unten = getCanvas(clean);

    VBox vbox = new VBox(oben, unten);

    vbox.setStyle("-fx-background-color: whitesmoke");

    stage.setScene(new Scene(vbox));
    stage.setTitle("KAO: Mittelwertfilter der Ordnung " + (App.multiParam - 1));
    stage.setResizable(false);
    stage.sizeToScene();
    stage.show();
  }

  private Canvas getCanvas(float[] curve) {
    Canvas canvas = new Canvas(BREITE, HOEHE / 2);
    var statistics = IntStream.range(0, curve.length)
      .mapToDouble(i -> curve[i])
      .summaryStatistics();
    double max = statistics.getMax(), min = statistics.getMin();
    double scale = HOEHE / 2 / (max - min) * 0.9;
    double[] x = new double[BREITE], y = new double[BREITE];
    Arrays.parallelSetAll(x, i -> i);
    Arrays.parallelSetAll(y, i -> curve[i] * scale + HOEHE / 4);
    GraphicsContext g = canvas.getGraphicsContext2D();
    g.setLineWidth(2); g.strokePolyline(x, y, BREITE);
    return canvas;
  }
}
